/*
**	File:		03 Middle of log growth.sql
**
**	Summary:	Run in a job to run indirect checkpoints prior to SQL 2012
**				set the percentage limit before running.
**
**	SQL Server Versions:	2005 onwards
**	----------------------------------------------------------------------------
**	Written by Rainer Unwin (c) 2014
**
**	For more scripts check out http://www.rmuniwn.com
**
**	You may alter this code for your own non-commercial purposes (e.g. in a
**	for-sale commercial tool).
**	You may republish altered code as long as you include this copyright and
**	give due credit, however you must obtain my prior permission before blogging
**	this code.
**
**	this code and information are provided "as is" without warranty of
**	any kind, either expressed or implied, including but not limited
**	to the implied warranties of merchantability and/or fitness for a
**	particular purpose. You use the script at your own risk and should
**	always test in a pre-production or test environment before running
**	in production.
*/
use master;
go

-- Remove any old databases
if exists (select name from sys.databases where name = N'tloggrowth') drop database tloggrowth;
go

-- Create database with same structure are before
create database tloggrowth
 ON  PRIMARY 
( NAME = N'tloggrowth', FILENAME = N'C:\SQLData\DataFiles\tloggrowth.mdf' , SIZE = 5120KB , FILEGROWTH = 1024KB )
 LOG ON 
( NAME = N'tloggrowth_log', FILENAME = N'C:\SQLData\LogFiles\tloggrowth_log.ldf' , SIZE = 4096KB , FILEGROWTH = 1024KB);
go
alter database tloggrowth modify file ( NAME = N'tloggrowth_log', SIZE = 8192KB );
go

-- Drop any object interfering in tempDB and create a #nums table with 3,000
-- incremental numbers to use
if object_id(N'tempdb..#nums', N'U') is not null drop table #nums;
go
create table #nums (i int not null primary key clustered);
go

;with nums (i)
as (
	select 1 union all select 1 union all select 1 union all select 1 union all select 1 union all
	select 1 union all select 1 union all select 1 union all select 1 union all select 1
)
insert into #nums (i)
select top (3000) row_number() over(order by n1.i)
from nums n1, nums n2, nums n3, nums n4;
go

use tloggrowth;
go

-- Create a table to use
create table grow (i int not null, col1 char(4500) default 'a' not null);
go

-- Check the current VLF structure, identical to before
dbcc loginfo(tloggrowth);
go

-- We need to move part way into the log
insert into grow (i)
select top (800) i
from #nums
order by i;
go

-- how much of the log is currently active
dbcc loginfo(tloggrowth);
go

-- Leave nothing to chance we want to be able to reuse the log
-- Simple recovery so checkpoint allows log clearing
checkpoint;
go

-- Show that we have now set part of the log as inactive
dbcc loginfo(tloggrowth);
go

-- Now we need to begin a long running transaction
begin tran

insert into grow (i)
select top (800) i
from #nums
where i > 800
order by i;

-- Show that we have now rolled round in the log, so far so good for the circular
-- nature of the log
dbcc loginfo(tloggrowth);

-- Show how log generate + log reserve (Mention this because we will grow the log
-- before it is actually full) is about 5 MB. We can also see where we are in the
-- log and where the transaction started.
select db_name(database_id) as database_name
	, cast((database_transaction_log_bytes_used / 1024.0) as numeric(19, 4)) as log_used_kb
	, cast((database_transaction_log_bytes_reserved / 1024.0) as numeric(19, 4)) as log_reserved_kb
	, database_transaction_begin_lsn
	, database_transaction_last_lsn
from sys.dm_tran_database_transactions
where database_id = db_id(N'tloggrowth');

-- Attempt log clearing (Won't work)
checkpoint;

dbcc loginfo(tloggrowth);

-- Why can we not clear the log (Our active transaction)
select name
	, log_reuse_wait_desc
from sys.databases
where name = N'tloggrowth';
go

-- This is another way to see where we started our transaction and also
-- that there is an open transaction
dbcc opentran;

-- Now lets put some more records in and grow the log
insert into grow (i)
select top (400) i
from #nums
where i > 1600
order by i;

-- Now that's weird right? We did use all the log (Remember the log reserve
-- well that's why). Also we now have the sub-optimal log situation (I
-- sometimes call this the spaghetti log).
dbcc loginfo(tloggrowth);

-- Now lets go and clear the log
commit;

checkpoint;

dbcc loginfo(tloggrowth);
go

-- Insert some more records
insert into grow (i)
select top (400) i
from #nums
where i > 2000
order by i;
go

-- Look at that, we didn't roll to the begining of the log from a VLF
-- position, in the file, perspective. We rolled to the oldest (FSeqNO)
-- VLF in the log. We now have 3 sections to our "Circular" log.
dbcc loginfo(tloggrowth);
go

-- Run the rest of this in one hit, explanation follows
if object_id(N'tempdb..#loginfo', N'U') is not null drop table #loginfo;
go
create table #loginfo (
	RecoveryUnitId		tinyint		not null
	, FileID			smallint	not null
	, FileSize			int			not null
	, StartOffset		bigint		not null
	, FSeqNo			int			not null
	, [Status]			tinyint		not null
	, Parity			tinyint		not null
	, CreateLSN			VARCHAR(50)	not null
);
go

-- drop RecoveryUnitId column if pre SQL Server 2012
declare @version varchar(30);
select @version = convert(varchar(30), serverproperty('productversion'), 2);
select @version = substring(@version, 1, charindex('.', @version) - 1 );

if (@version < 11)
	alter table #loginfo drop column RecoveryUnitId;
go

insert into #loginfo
exec(N'dbcc loginfo(tloggrowth);');
go

select StartOffset
	, FSeqNo
	, [Status]
	, Parity
	, row_number() over(order by StartOffset) as PositionInDataFile
	, row_number() over(order by FSeqNo) as PositionOfVLFs
from #loginfo
order by StartOffset;
go

if object_id(N'tempdb..#loginfo', N'U') is not null drop table #loginfo;
go
if object_id(N'tempdb..#nums', N'U') is not null drop table #nums;
go
-- What we can see here is the startOffset matches the PositionInDataFile
-- ordering and the FSeqNo matches the PositionOfVLFs ordering. These no
-- longer match, like the nice circular log from the last demo.
-- Also notice that there are 12 VLFs so we will move to VLF 38 (Or
-- PositionOfVLFs = 1) next. So we have 3 sections to our log as below:
--	- VLFs 11 - 4
--	- VLFs 5 and 6
--	- VLFs 7 - 10
-- Of these we effectively go through these sections in order but go
-- through the sections backwards.