/*
**	File:		06 Truncate log and efficient logging.sql
**
**	Summary:	Run in a job to run indirect checkpoints prior to SQL 2012
**				set the percentage limit before running.
**
**	SQL Server Versions:	2005 onwards
**	----------------------------------------------------------------------------
**	Written by Rainer Unwin (c) 2014
**
**	For more scripts check out http://www.rmuniwn.com
**
**	You may alter this code for your own non-commercial purposes (e.g. in a
**	for-sale commercial tool).
**	You may republish altered code as long as you include this copyright and
**	give due credit, however you must obtain my prior permission before blogging
**	this code.
**
**	this code and information are provided "as is" without warranty of
**	any kind, either expressed or implied, including but not limited
**	to the implied warranties of merchantability and/or fitness for a
**	particular purpose. You use the script at your own risk and should
**	always test in a pre-production or test environment before running
**	in production.
*/
use [tlogbasics];
go

-- prep a table to use
if object_id(N'trunks', N'U') is not null drop table trunks;
go
create table trunks (i int identity, col1 char(4500) not null default 'a');
go

-- Run this transaction in one go to see how much log is generated
begin tran

	insert into trunks (col1)
	select top (1000) 'a'
	from (
		select top(1000) name
		from sys.all_columns
	) v (name);

	select db_name(database_id) as database_name
		, cast((database_transaction_log_bytes_used / 1024.0) as numeric(19, 4)) as log_used_kb
	from sys.dm_tran_database_transactions
	where database_id = db_id(N'tlogbasics');

commit;
-- Note the amount of log generated (Abount 5 MB)

select count(*)
from dbo.trunks;

-- Now truncate the table, run in one go and not the amount of log to truncate
begin tran;

	truncate table trunks;

	-- show empty table
	select *
	from trunks;

	select db_name(database_id) as database_name
		, cast((database_transaction_log_bytes_used / 1024.0) as numeric(19, 4)) as log_used_kb
	from sys.dm_tran_database_transactions
	where database_id = db_id(N'tlogbasics');

rollback tran;
-- Amount of log to truncate is a LOT less (About 29 KB)
-- This is about 1/175th the amount of log to create the records

-- show that truncate was rolled back
select *
from trunks;



/* ********************************************************************************
** This next section is provided as an advanced topic and should be attempted when
** you are comfortable with how the log works and want to stretch your learning to
** the next level. Feel free to ask questions (Rainer@RMUnwin.com) although it may
** take a few days to get a response :)
**
** Also please note that a decent understanding of allocation unit pages is very
** helpful here. Please read up on Page Free Space (PFS), Global Allocation Map
** (GAM) and Shared Global Allocation Map (SGAM) pages if you are not comfortable
** with these topics. Paul Randal (Highly recommended reading for internals) has
** some great articles on these on his blog 
** http://www.sqlskills.com/blogs/paul/category/inside-the-storage-engine/
*/

-- In case you want to have a look at the records I suggest using the below commands
-- this is a much more in depth topic and is approaching level 400/450. Some pointers
-- are provided below to help out.

truncate table trunks;
go

-- Get the current LSN to use in fn_dblog, you will be doing this a few times

-- Run this command and then paste the result into the variable below
-- then run the below declare and statement, save the result which you will need next
select top (1) [Current LSN]
from fn_dblog(null, null)
order by [Current LSN] desc;
go

declare @lsn varchar(50) = ''

select  cast( cast( convert( varbinary, Part1, 2 ) as int ) as varchar ) +
       right( '0000000000' + cast( cast( convert( varbinary, Part2, 2 ) as int ) as varchar ), 10 ) +
       right( '00000'      + cast( cast( convert( varbinary, Part3, 2 ) as int ) as varchar ), 5 ) as [Converted LSN]
       from (select left( @lsn, 8 ) as Part1,
       substring( @lsn, 10, 8 ) as Part2,
       right( @lsn, 4 )         as Part3) s;
go
-- Make a note of the result: 
-- You will now run the below command and check what is in the log
-- paste the above value over the comment in the fn_dblog command
insert into trunks (col1)
select top (1000) 'a'
from (
	select top(1000) name
	from sys.all_columns
) v (name);

select Operation
	, Context -- PFS = page free space page, heap = dbo.trunks, GAM = global allocation map, SGAM = shared global allocation map
	, [Log Record Length] -- value in bytes
	, [Log Reserve] -- value in bytes
	, [AllocUnitName]
from ::fn_dblog(/* paste value here */, null)
order by [Current LSN];
-- Note that there are about 8,400 records displayed

-- Run this command and then paste the result into the variable below
-- then run the below declare and statement, save the result which you will need next
select top (1) [Current LSN]
from fn_dblog(null, null)
order by [Current LSN] desc;
go

declare @lsn varchar(50) = ''

select  cast( cast( convert( varbinary, Part1, 2 ) as int ) as varchar ) +
       right( '0000000000' + cast( cast( convert( varbinary, Part2, 2 ) as int ) as varchar ), 10 ) +
       right( '00000'      + cast( cast( convert( varbinary, Part3, 2 ) as int ) as varchar ), 5 ) as [Converted LSN]
       from (select left( @lsn, 8 ) as Part1,
       substring( @lsn, 10, 8 ) as Part2,
       right( @lsn, 4 )         as Part3) s;
go
-- Make a note of the result: 

-- You will now run the below command and check what is in the log
-- paste the above value over the comment in the fn_dblog command
truncate table trunks;
go

select Operation
	, Context -- PFS = page free space page, heap = dbo.trunks, GAM = global allocation map, SGAM = shared global allocation map
	, [Log Record Length] -- value in bytes
	, [Log Reserve] -- value in bytes
	, [AllocUnitName]
from ::fn_dblog(/* paste value here */, null)
order by [Current LSN];
-- Note that there are less than 400 records, they are also a lot smaller than before.
-- There is a lot less work for SQL Server to do here because truncate just deallocates
-- the pages of data, there is no delete done. These values are held locked and can be
-- rolled back so the restore is almost instantaneous because the allocation unit pages
-- are just changed back.